static char * sccsid = "@(#)  walk.c  (v1.2 3/21/90)";

/*
 * Do a depth first walk on tree.
 */

# include <stdio.h>
# include "kef.h"

#ifdef DEBUG_WALK
#	define L_DEBUG
#else
#	ifdef L_DEBUG
#		undef L_DEBUG
#	endif
#endif
# include "local.h"

	int
kef_walk (kdb, inx)
	struct kef_db * kdb;
	kef_index * inx;
/*
 * Walk depth-first through tree of character cells.
 * It must be possible to build string from this information so
 * this routine sets fields "tag" which are unused execpt in cleanup
 * to point at father node. String can be built backwards by running
 * up chain of "tag" links. The "tag" stuff also serves as a stack
 * so we don't have to hold a stack to retrace our steps on the tree.
 * Returns normal KEF error codes.
 * If "*inx == KEF_NIL" on start, we start at the top of the tree.
 * At end : the end of the tree is reached, when "*inx == KEF_NIL"
 */
{
	DI("kef_walk ::")
	register kef_index at, up;
	register struct kef_node * nd;

	DEB2("%s started at node %1d\n", *inx);
	nd = kdb->node;
	if (*inx == KEF_NIL) {
		at = kdb->root;
		nd[at].tag = KEF_NIL;		/* put an end for retracing */
	} else
		at = *inx;
	DEB2("%s current node = %1d\n", at);

	/*
	 * Continue walk through tree.
	 * First try to go down.
	 */
	if (nd[at].sub != KEF_NIL) {
		/*
		 * Can go down.
		 */
		up = at;
		at = nd[at].sub;
		nd[at].tag = up;		/* set pointer to parent node */
		DEB2("%s stepped down to %1d\n", at);
	} else {
		while (at != kdb->root && nd[at].next == KEF_NIL) {
			at = nd[at].tag;
			DEB2 ("%s retracing to %1d\n", at);
		}
		if (at != kdb->root) {
			DEB2 ("%s go right to %1d\n", at);
			up = nd[at].tag;
			at = nd[at].next;
			nd[at].tag = up;		/* set pointer to parent node */
		} else
			at = KEF_NIL;
	}
	*inx = at;
	DEB2("%s return node index %1d\n", at);
	return KEF_E_OK;
}

	int
kef_get_value (kdb, pos, val)
	struct kef_db * kdb;
	kef_index pos;
	kef_value * val;
/*
 * Returns 1 if value, 0 else.
 * Sets val to value.
 */
{
	DI("kef_get_value ::")

	if (HAS_VALUE(kdb->node[pos])) {
		DEB3("%s value at node %1d : %1d\n", pos, kdb->node[pos].value);
		*val = kdb->node[pos].value;
		return 1;
	} else {
		DEB2("%s no value at node %1d\n", pos);
		return 0;
	}
}
