static char * sccsid = "@(#)  write.c  (v1.6 3/15/90)";

/*
 * Write a kdb structure to file.
 *
 * File structure:
 * - header
 * - kdb structure padded to fit yield 4 byte alignment. This is because
 *   If a compiler chooses to rearrange the fields, we are licked, though.
 * - nodes (must be 2 byte aligned)
 *
 * Conversion (swap) is done on read.
 * Don't use fwrite/fread : they may introduce malloc calls by including
 * _flsbuf (the routine used by stdin to fill its buffers).
 */

# include <stdio.h>
# include "kef.h"

#ifdef DEBUG_IO
#	define L_DEBUG
#else
#	ifdef L_DEBUG
#		undef L_DEBUG
#	endif
#endif
# include "local.h"

	int
kef_write(fd, kdb)
	int fd;
	struct kef_db * kdb;
/*
 * Write a kdb structure to file fd.
 *
 * Format on file:
 * - num_nodes
 * - kdb structure
 * - nodes
 */
{
	DI("kef_write ::")
	int n;
	struct kef_fheader kfh;

	DEB1("%s called \n");

	kfh.magic[0] = KEF_MAGIC0;
	kfh.magic[1] = KEF_MAGIC1;
	kfh.magic[2] = KEF_MAGIC2;
	kfh.magic[3] = KEF_MAGIC3;
	/*
	 * Set swap to value 0x0201. On little endian machines this
	 * will be written to a file as 0102, on big endian machines it
	 * will be 0201.
	 */
	kfh.swap = 0x0201;
	kfh.num_nodes = kdb->high_nodes;
	kfh.size_kdb = (int)(&(kdb->node)) - (int)(kdb);
	kfh.size_node = sizeof (struct kef_node);

# ifdef L_DEBUG
	dump_fheader (prot, &kfh);
# endif

	if (write (fd, &kfh, sizeof(kfh)) != sizeof(kfh))
		return KEF_E_WRITE_ERROR;
	DEB2("%s wrote header with %1d characters\n", sizeof(kfh));

	/*
	 * Write kef_db.
	 */
	n = kfh.size_kdb;
	if (write (fd, kdb, n) != n)
		return KEF_E_WRITE_ERROR;
	DEB2("%s wrote kef_db struct with %1d characters\n", n);

	if (n & 0x03) {
		/*
		 * Pad the kef_db structure.
		 */
		n = 4 - (n & 0x03);
		if (write (fd, kdb, n) != n)
			return KEF_E_WRITE_ERROR;
		DEB2("%s wrote padding %1d characters\n", n);
	}

	n = kfh.num_nodes * sizeof (struct kef_node);
	if (write (fd, kdb->node, n) != n)
		return KEF_E_WRITE_ERROR;
	DEB2("%s wrote node total of %1d characters\n", n);

	DEB1("%s done.\n");

	return KEF_E_OK;
}
